# -*- coding: utf-8 -*-

mapping = (
    u"abvgdezijklmnoprstufhcABVGDEZIJKLMNOPRSTUFHC",
    u"абвгдезијклмнопрстуфхцАБВГДЕЗИЈКЛМНОПРСТУФХЦ",
)

reversed_specific_mapping = (
    u"",
    u""
)

pre_processor_mapping = {
    u"gj": u"ѓ",
    u"zh": u"ж",
    u"dz": u"ѕ",
    u"lj": u"љ",
    u"nj": u"њ",
    u"кј": u"ќ",
    u"ch": u"ч",
    u"dzh": u"џ",
    u"sh": u"ш",
    u"Gj": u"Ѓ",
    u"Zh": u"Ж",
    u"Dz": u"Ѕ",
    u"Lj": u"Љ",
    u"Nj": u"Њ",
    u"Kj": u"Ќ",
    u"Ch": u"Ч",
    u"Dzh": u"Џ",
    u"Sh": u"Ш",
}



# -*- coding: utf-8 -*-

__title__ = 'transliterate.contrib.languages.ka.translit_language_pack'
__author__ = 'Artur Barseghyan'
__copyright__ = '2013-2015 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'
__all__ = ('GeorgianLanguagePack',)

from transliterate.base import TranslitLanguagePack, registry
from transliterate.contrib.languages.ka import data

class GeorgianLanguagePack(TranslitLanguagePack):
    """
    Language pack for Georgian language. See
    `http://en.wikipedia.org/wiki/Georgian_alphabet for details.
    """
    language_code = "ka"
    language_name = "Georgian"
    character_ranges = ((0x10A0, 0x10C5), (0x10D0, 0x10FC), (0x2D00, 0x2D25))
    mapping = data.mapping
    pre_processor_mapping = data.pre_processor_mapping
    detectable = True


registry.register(GeorgianLanguagePack)



# -*- coding: utf-8 -*-

mapping = (
    "abvgdjziklmnouprstuufheABVGDJZIKLMNOUPRSTUUFHE",
    "абвгджзиклмноөпрстуүфхэАБВГДЖЗИКЛМНОӨПРСТУҮФХЭ",
)
reversed_specific_mapping = (
    "ъьЪЬйЙ",
    "iiIIiI"
)
pre_processor_mapping = {
    "ii": "ы",
    "II": "Ы",
    "ye": "е",
    "YE": "Е",
    "yo": "ё",
    "YO": "Ё",
    "ts": "ц",
    "TS": "Ц",
    "ch": "ч",
    "CH": "Ч",
    "sh": "ш",
    "SH": "Ш",
    "sh": "щ",
    "SH": "Щ",
    "yu": "ю",
    "YU": "Ю",
    "ya": "я",
    "YA": "Я",
}



from transliterate.helpers import PY32

if PY32:
    from transliterate.tests.data.python32 import *
else:
    from transliterate.tests.data.normal import *

__title__ = 'transliterate.tests.data'
__author__ = 'Artur Barseghyan'
__copyright__ = 'Copyright (c) 2013-2016 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'



# -*- coding: utf-8 -*-

mapping = (
    u"abvgdjziklmnouprstuufheABVGDJZIKLMNOUPRSTUUFHE",
    u"абвгджзиклмноөпрстуүфхэАБВГДЖЗИКЛМНОӨПРСТУҮФХЭ",
)
reversed_specific_mapping = (
    u"ъьЪЬйЙ",
    u"iiIIiI"
)
pre_processor_mapping = {
    u"ii": u"ы",
    u"II": u"Ы",
    u"ye": u"е",
    u"YE": u"Е",
    u"yo": u"ё",
    u"YO": u"Ё",
    u"ts": u"ц",
    u"TS": u"Ц",
    u"ch": u"ч",
    u"CH": u"Ч",
    u"sh": u"ш",
    u"SH": u"Ш",
    u"sh": u"щ",
    u"SH": u"Щ",
    u"yu": u"ю",
    u"YU": u"Ю",
    u"ya": u"я",
    u"YA": u"Я",
}



# -*- coding: utf-8 -*-

mapping = (
    "abgdezilkhmjnpsvturqoABGDEZILKHMJNPSVTURQO",
    "აბგდეზილქჰმჯნფსვთურყოႠႡႢႣႤႦႨႪႵჀႫႿႼႴႱႥႧႳႰႷႭ",
)
pre_processor_mapping = {
    "k'": "კ",
    "p'": "პ",
    "zh'": "ჟ",
    "t'": "ტ",
    "gh": "ღ",
    "sh": "შ",
    "ch": "ჩ",
    "ts": "ც",
    "dz": "ძ",
    "ts'": "წ",
    "ch'": "ჭ",
    "kh": "ხ",
    "K'": "Ⴉ",
    "P'": "Ⴎ",
    "Zh'": "Ⴏ",
    "T'": "Ⴒ",
    "Gh": "Ⴖ",
    "Sh": "Ⴘ",
    "Ch": "Ⴙ",
    "Ts": "Ⴚ",
    "Dz": "Ⴛ",
    "Ts'": "Ⴜ",
    "Ch'": "Ⴝ",
    "Kh": "Ⴞ",
}



__title__ = 'transliterate.defaults'
__author__ = 'Artur Barseghyan'
__copyright__ = '2013-2015 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'
__all__ = (
    'LANGUAGES_DIR', 'CONTRIB_DIR', 'LANGUAGE_PACK_MODULE_NAME',
    'LANGUAGE_DETECTION_MAX_NUM_KEYWORDS', 'DEBUG',
)

LANGUAGES_DIR = ('contrib', 'languages')

CONTRIB_DIR =  ('contrib', 'apps')
LANGUAGE_PACK_MODULE_NAME = 'translit_language_pack'
LANGUAGE_DETECTION_MAX_NUM_KEYWORDS = 16

DEBUG = False



__title__ = 'transliterate.helpers'
__author__ = 'Artur Barseghyan'
__copyright__ = '2013-2015 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'
__all__ = ('PROJECT_DIR', 'PY32',)

import os
import sys

try:
    PY32 = (sys.version_info[0] == 3 and sys.version_info[1] == 2)
except:
    PY32 = False

PROJECT_DIR = lambda base: os.path.abspath(
    os.path.join(
        os.path.dirname(__file__),
        (os.path.join(*base) if isinstance(base, (list, tuple)) else base)
    ).replace('\\','/')
)

#PROJECT_DIR = lambda base : os.path.abspath(os.path.join(os.path.dirname(__file__), base).replace('\\','/'))



# -*- coding: utf-8 -*-

mapping = (
    u"abvhgdezyijklmnoprstuf'ABVHGDEZYIJKLMNOPRSTUF'",
    u"абвгґдезиійклмнопрстуфьАБВГҐДЕЗИІЙКЛМНОПРСТУФЬ",
)
reversed_specific_mapping = (
    u"ьЬ",
    u"''"
)
pre_processor_mapping = {
    u"ye": u"є",
    u"zh": u"ж",
    u"yi": u"ї",
    u"kh": u"х",
    u"ts": u"ц",
    u"ch": u"ч",
    u"sh": u"ш",
    u"shch": u"щ",
    u"ju": u"ю",
    u"ja": u"я",
    u"Ye": u"Є",
    u"Zh": u"Ж",
    u"Yi": u"Ї",
    u"Kh": u"Х",
    u"Ts": u"Ц",
    u"Ch": u"Ч",
    u"Sh": u"Ш",
    u"Shch": u"Щ",
    u"Ju": u"Ю",
    u"Ja": u"Я"
}


from transliterate.utils import (
    translit, get_available_language_codes, detect_language, slugify
)
from transliterate.utils import get_available_language_packs

__title__ = 'transliterate'
__version__ = '1.8'
__build__ = 0x000019
__author__ = 'Artur Barseghyan'
__copyright__ = '2013-2016 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'
__all__ = (
    'translit', 'get_available_language_codes', 'detect_language', 'slugify',
    'get_available_language_packs',
)



# -*- coding: utf-8 -*-

mapping = (
        "abgdezilxkhmjnpsvtrcq&ofABGDEZILXKHMJNPSVTRCQOF",
        "աբգդեզիլխկհմյնպսվտրցքևօֆԱԲԳԴԵԶԻԼԽԿՀՄՅՆՊՍՎՏՐՑՔՕՖ",
    )
reversed_specific_mapping = (
    "ռՌ",
    "rR"
)
reversed_specific_pre_processor_mapping = {
    "ու": "u",
    "Ու": "U"
}
pre_processor_mapping = {
    # lowercase
    "e'": "է",
    "y": "ը",
    "th": "թ",
    "jh": "ժ",
    "ts": "ծ",
    "dz": "ձ",
    "gh": "ղ",
    "tch": "ճ",
    "sh": "շ",
    "vo": "ո",
    "ch": "չ",
    "dj": "ջ",
    "ph": "փ",
    "u": "ու",

    # uppercase
    "E'": "Է",
    "Y": "Ը",
    "Th": "Թ",
    "Jh": "Ժ",
    "Ts": "Ծ",
    "Dz": "Ձ",
    "Gh": "Ղ",
    "Tch": "Ճ",
    "Sh": "Շ",
    "Vo": "Ո",
    "Ch": "Չ",
    "Dj": "Ջ",
    "Ph": "Փ",
    "U": "Ու"
}



# -*- coding: utf-8 -*-

mapping = (
    u"abvgdezijklmnoprstufh'y'ABVGDEZIJKLMNOPRSTUFH'Y'",
    u"абвгдезийклмнопрстуфхъыьАБВГДЕЗИЙКЛМНОПРСТУФХЪЫЬ",
)
reversed_specific_mapping = (
    u"ёэЁЭъьЪЬ",
    u"eeEE''''"
)
pre_processor_mapping = {
    u"zh": u"ж",
    u"ts": u"ц",
    u"ch": u"ч",
    u"sh": u"ш",
    u"sch": u"щ",
    u"ju": u"ю",
    u"ja": u"я",
    u"Zh": u"Ж",
    u"Ts": u"Ц",
    u"Ch": u"Ч",
    u"Sh": u"Ш",
    u"Sch": u"Щ",
    u"Ju": u"Ю",
    u"Ja": u"Я"
}



# -*- coding: utf-8 -*-

__title__ = 'transliterate.contrib.languages.mk.translit_language_pack'
__author__ = 'Igor Stamatovski'
__copyright__ = '2016 Igor Stamatovski'
__license__ = 'GPL 2.0/LGPL 2.1'
__all__ = ('MacedonianLanguagePack',)

from transliterate.base import TranslitLanguagePack, registry
from transliterate.contrib.languages.mk import data

class MacedonianLanguagePack(TranslitLanguagePack):
    """
    Language pack for Macedonian language.
    See http://en.wikipedia.org/wiki/Romanization_of_Macedonian for details.
    """
    language_code = "mk"
    language_name = "Macedonian"
    character_ranges = ((0x0400, 0x04FF), (0x0500, 0x052F))
    mapping = data.mapping
    reversed_specific_mapping = data.reversed_specific_mapping
    pre_processor_mapping = data.pre_processor_mapping
    detectable = False


registry.register(MacedonianLanguagePack)



# -*- coding: utf-8 -*-

mapping = (
    "abvgdezijklmnoprstufh'y'ABVGDEZIJKLMNOPRSTUFH'Y'",
    "абвгдезийклмнопрстуфхъыьАБВГДЕЗИЙКЛМНОПРСТУФХЪЫЬ",
)
reversed_specific_mapping = (
    "ёэЁЭъьЪЬ",
    "eeEE''''"
)
pre_processor_mapping = {
    "zh": "ж",
    "ts": "ц",
    "ch": "ч",
    "sh": "ш",
    "sch": "щ",
    "ju": "ю",
    "ja": "я",
    "Zh": "Ж",
    "Ts": "Ц",
    "Ch": "Ч",
    "Sh": "Ш",
    "Sch": "Щ",
    "Ju": "Ю",
    "Ja": "Я"
}



# -*- coding: utf-8 -*-

latin_text = u"Lorem ipsum dolor sit amet"
armenian_text = u'Լօրեմ իպսում դօլօր սիտ ամետ'
cyrillic_text = u'Лорем ипсум долор сит амет'
ukrainian_cyrillic_text = u'Лорем іпсум долор сіт амет'
bulgarian_cyrillic_text = u'Лорем ипсум долор сит амет'
georgian_text = u'Ⴊორემ იფსუმ დოლორ სით ამეთ'
greek_text = u'Λορεμ ιψυμ δολορ σιτ αμετ'
hebrew_text = u'Lורeמ יpסuמ דולור סית אמeת'
mongolian_cyrillic_text = u'Лорэм ипсүм долор сит амэт'

test_15_register_custom_language_pack_mapping = (
    u"abcdefghij",
    u"1234567890",
)

test_33_register_unregister_mapping = (
    u"abcdefghij",
    u"1234567890",
)

test_34_latin_to_latin_mapping = (
    u"abgdezilxkhmjnpsvtrcqw&ofABGDEZILXKHMJNPSVTRCQOFW",
    u"zbgdeailxkhnjmpswtrcqv&ofZBGDEAILXKHNJMPSWTRCQOFV",
)

test_34_latin_to_latin_characters = u"abgdezilxkhmjnpsvtrcqw&ofABGDEZILXKHMJNPSVTRCQOFW"

test_34_latin_to_latin_reversed_characters = u"abgdezilxkhmjnpsvtrcqw&ofABGDEZILXKHMJNPSVTRCQOFW"

test_34_latin_to_latin_text = u"Lorem ipsum dolor sit amet 123453254593485938"

__title__ = 'transliterate.tests.data.normal'
__author__ = 'Artur Barseghyan'
__copyright__ = '2013-2016 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'



# -*- coding: utf-8 -*-

import unicodedata
import re

import six

from transliterate.exceptions import (
    ImproperlyConfigured, InvalidRegistryItemType
)

__title__ = 'transliterate.base'
__author__ = 'Artur Barseghyan'
__copyright__ = 'Copyright (c) 2013-2016 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'
__all__ = ('TranslitLanguagePack', 'registry',)

class TranslitLanguagePack(object):
    """
    Base language pack. The attributes below shall be defined in every
    language pack.

    ``language_code``: Language code (obligatory). Example value: 'hy', 'ru'.
    ``language_name``: Language name (obligatory). Example value: 'Armenian',
        'Russian'.
    ``character_ranges``: Character ranges that are specific to the language.
        When making a pack, check `this
        <http://en.wikipedia.org/wiki/List_of_Unicode_characters>`_ page for
        the ranges.
    ``mapping``: Mapping  (obligatory). A tuple, consisting of two strings
        (source and target). Example value: (u'abc', u'աբց').
    ``reversed_specific_mapping``: Specific mapping (one direction only) used
        when transliterating from target script to source script (reversed
        transliteration).
    ՝՝pre_processor_mapping՝՝: Pre processor mapping (optional). A dictionary
        mapping for letters that can't be represented by a single latin letter.
    ՝՝reversed_specific_pre_processor_mapping՝՝: Pre processor mapping (
        optional). A dictionary mapping for letters that can't be represented
        by a single latin letter (reversed transliteration).

    :example:
>>>    class ArmenianLanguagePack(TranslitLanguagePack):
>>>    language_code = "hy"
>>>    language_name = "Armenian"
>>>    character_ranges = ((0x0530, 0x058F), (0xFB10, 0xFB1F))
>>>    mapping = (
>>>        u"abgdezilxkhmjnpsvtrcq&ofABGDEZILXKHMJNPSVTRCQOF", # Source script
>>>        u"աբգդեզիլխկհմյնպսվտրցքևօֆԱԲԳԴԵԶԻԼԽԿՀՄՅՆՊՍՎՏՐՑՔՕՖ", # Target script
>>>    )
>>>    reversed_specific_mapping = (
>>>        u"ռՌ",
>>>        u"rR"
>>>    )
>>>    pre_processor_mapping = {
>>>        # lowercase
>>>        u"e'": u"է",
>>>        u"y": u"ը",
>>>        u"th": u"թ",
>>>        u"jh": u"ժ",
>>>        u"ts": u"ծ",
>>>        u"dz": u"ձ",
>>>        u"gh": u"ղ",
>>>        u"tch": u"ճ",
>>>        u"sh": u"շ",
>>>        u"vo": u"ո",
>>>        u"ch": u"չ",
>>>        u"dj": u"ջ",
>>>        u"ph": u"փ",
>>>        u"u": u"ու",
>>>
>>>        # uppercase
>>>        u"E'": u"Է",
>>>        u"Y": u"Ը",
>>>        u"Th": u"Թ",
>>>        u"Jh": u"Ժ",
>>>        u"Ts": u"Ծ",
>>>        u"Dz": u"Ձ",
>>>        u"Gh": u"Ղ",
>>>        u"Tch": u"Ճ",
>>>        u"Sh": u"Շ",
>>>        u"Vo": u"Ո",
>>>        u"Ch": u"Չ",
>>>        u"Dj": u"Ջ",
>>>        u"Ph": u"Փ",
>>>        u"U": u"Ու"
>>>    }
>>>    reversed_specific_pre_processor_mapping = {
>>>        u"ու": u"u",
>>>        u"Ու": u"U"
>>>    }
    Note, that in Python 3 you won't be using u prefix before the strings.
    """
    language_code = None
    language_name = None
    character_ranges = None
    mapping = None
    reversed_specific_mapping = None

    reversed_pre_processor_mapping_keys = []

    reversed_specific_pre_processor_mapping = None
    reversed_specific_pre_processor_mapping_keys = []

    pre_processor_mapping = None
    pre_processor_mapping_keys = []

    detectable = False
    characters = None
    reversed_characters = None

    def __init__(self):
        try:
            assert self.language_code is not None
            assert self.language_name is not None
            assert self.mapping
        except Exception as e:
            raise ImproperlyConfigured(
                "You should define ``language_code``, ``language_name`` and "
                "``mapping`` properties in your subclassed "
                "``TranslitLanguagePack`` class.")
        super(TranslitLanguagePack, self).__init__()

        # Creating a translation table from the mapping set.
        self.translation_table = {}
        [self.translation_table.update({ord(a): ord(b)})
         for a, b in zip(*self.mapping)]

        # Creating a reversed translation table.
        self.reversed_translation_table = dict(
            zip(self.translation_table.values(), self.translation_table.keys())
        )

        # If any pre-processor rules defined, reversing them for later use.
        if self.pre_processor_mapping:
            self.reversed_pre_processor_mapping = dict(
                zip(
                    self.pre_processor_mapping.values(),
                    self.pre_processor_mapping.keys())
                )
            self.pre_processor_mapping_keys = self.pre_processor_mapping.keys()
            self.reversed_pre_processor_mapping_keys = \
                self.pre_processor_mapping.values()

        else:
            self.reversed_pre_processor_mapping = None

        if self.reversed_specific_mapping:
            self.reversed_specific_translation_table = {}
            [self.reversed_specific_translation_table.update({ord(a): ord(b)})
             for a, b in zip(*self.reversed_specific_mapping)]

        if self.reversed_specific_pre_processor_mapping:
            self.reversed_specific_pre_processor_mapping_keys = \
                self.reversed_specific_pre_processor_mapping.keys()

        self._characters = '[^]'

        if self.characters is not None:
            self._characters = '[^{0}]'.format('\\'.join(list(self.characters)))

        self._reversed_characters = '[^]'
        if self.reversed_characters is not None:
            self._reversed_characters = \
                '[^{0}]'.format('\\'.join(list(self.characters)))

    def translit(self, value, reversed=False, strict=False, fail_silently=True):
        """
        Transliterates the given value according to the rules set in the
        transliteration pack.

        :param str value:
        :param bool reversed:
        :param bool strict:
        :return str:
        """
        if not six.PY3:
            value = unicode(value)

        if reversed:
            # Handling reversed specific translations (one side only).
            if self.reversed_specific_mapping:
                value = value.translate(
                    self.reversed_specific_translation_table
                )

            if self.reversed_specific_pre_processor_mapping:
                for rule in self.reversed_specific_pre_processor_mapping_keys:
                    value = value.replace(
                        rule,
                        self.reversed_specific_pre_processor_mapping[rule]
                    )

            # Handling pre-processor mappings.
            if self.reversed_pre_processor_mapping:
                for rule in self.reversed_pre_processor_mapping_keys:
                    value = value.replace(
                        rule,
                        self.reversed_pre_processor_mapping[rule]
                    )

            return value.translate(self.reversed_translation_table)

        if self.pre_processor_mapping:
            for rule in self.pre_processor_mapping_keys:
                value = value.replace(rule, self.pre_processor_mapping[rule])
        res = value.translate(self.translation_table)

        if strict:
            res = self._make_strict(value=res,
                                    reversed=reversed,
                                    fail_silently=fail_silently)

        return res

    def _make_strict(self, value, reversed=False, fail_silently=True):
        """
        Strips out unnecessary characters from the string.

        :param string value:
        :param bool reversed:
        :param bool fail_silently:
        :return string:
        """
        try:
            return self.make_strict(value, reversed)
        except Exception as e:
            if fail_silently:
                return value
            else:
                #import ipdb; ipdb.set_trace()
                raise e

    def make_strict(self, value, reversed=False):
        """
        Strips out unnecessary characters from the string.

        :param string value:
        :param bool reversed:
        :return string:
        """
        if reversed:
            if self.reversed_characters:
                # Make strict based on the ``reversed_characters``.
                value = re.sub(self._reversed_characters, '', value)
            else:
                # Make strict based on the ``character_ranges`` specified.
                pass
        else:
            if self.characters:
                # Make strict based on the ``characters``.
                value = re.sub(self._characters, '', value)
            else:
                # Make strict based on the ``character_ranges`` specified.
                pass

        return value

    @classmethod
    def contains(cls, character):
        """
        Checks if given character belongs to the language pack.

        :return bool:
        """
        if cls.character_ranges:
            char_num = unicodedata.normalize('NFC', character)
            char_num = ord(char_num)
            for character_range in cls.character_ranges:
                range_lower = character_range[0]
                range_upper = character_range[1]
                if char_num >= range_lower and char_num <= range_upper:
                    return True
        return False

    def suggest(value, reversed=False, limit=None):
        """
        Suggest possible variants (some sort of auto-complete).

        :param str value:
        :param int limit: Limit number of suggested variants.
        :return list:
        """
        # TODO

    def detect(text, num_words=None):
        """
        Heavy language detection, which is activated for languages that are
        harder detect (like Russian Cyrillic and Ukrainian Cyrillic).

        :param unicode value: Input string.
        :param int num_words: Number of words to base decision on.
        :return bool: True if detected and False otherwise.
        """
        # TODO


class TranslitRegistry(object):
    """
    Language pack registry.
    """
    def __init__(self):
        self._registry = {}
        self._forced = []

    def register(self, cls, force=False):
        """
        Registers the language pack in the registry.

        :param transliterate.base.LanguagePack cls: Subclass of
            ``transliterate.base.LanguagePack``.
        :param bool force: If set to True, item stays forced. It's not possible
            to unregister a forced item.
        :return bool: True if registered and False otherwise.
        """
        if not issubclass(cls, TranslitLanguagePack):
            raise InvalidRegistryItemType(
                "Invalid item type `{0}` for registry "
                "`{1}`".format(cls, self.__class__)
            )

        # If item has not been forced yet, add/replace its' value in the
        # registry.
        if force:

            if not cls.language_code in self._forced:
                self._registry[cls.language_code] = cls
                self._forced.append(cls.language_code)
                return True
            else:
                return False

        else:

            if cls.language_code in self._registry:
                return False
            else:
                self._registry[cls.language_code] = cls
                return True

    def unregister(self, cls):
        """
        Unregisters an item from registry.

        :param transliterate.base.LanguagePack cls: Subclass of
            ``transliterate.base.LanguagePack``.
        :return bool: True if unregistered and False otherwise.
        """
        if not issubclass(cls, TranslitLanguagePack):
            raise InvalidRegistryItemType(
                "Invalid item type `{0}` for registry "
                "`{1}`".format(cls, self.__class__)
            )

        # Only non-forced items are allowed to be unregistered.
        if cls.language_code in self._registry \
                and not cls.language_code in self._forced:

            self._registry.pop(cls.language_code)
            return True
        else:
            return False

    def get(self, language_code, default=None):
        """
        Gets the given language pack from the registry.

        :param str language_code:
        :return transliterate.base.LanguagePack: Subclass of
            ``transliterate.base.LanguagePack``.
        """
        return self._registry.get(language_code, default)


# Register languages by calling registry.register()
registry = TranslitRegistry()



__title__ = 'transliterate.tests.helpers'
__author__ = 'Artur Barseghyan'
__copyright__ = '2013-2015 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'

import six
from six import print_

from transliterate.tests.defaults import PRINT_INFO

def print_info(func):
    """
    Prints some useful info.
    """
    if not PRINT_INFO:
        return func

    def inner(self, *args, **kwargs):
        result = func(self, *args, **kwargs)

        print_('\n{0}'.format(func.__name__))
        print_('============================')
        print_('""" {0} """'.format(func.__doc__.strip()))
        print_('----------------------------')
        if result is not None:
            try:
                print_(result)
            except Exception as e:
                print_(result.encode('utf8'))

        return result
    return inner


def py2only(func):
    """
    Skips the test on Python 3.
    """
    if not six.PY3:
        return func

    def dummy(self, *args, **kwargs):
        pass

    return dummy



# -*- coding: utf-8 -*-

mapping = (
    "avgdeziklmnxoprstyfAVGDEZIKLMNXOPRSTYF",
    "αβγδεζικλμνξοπρστυφΑΒΓΔΕΖΙΚΛΜΝΞΟΠΡΣΤΥΦ",
)
reversed_specific_mapping = (
    "ςάέήίύόώϊϋΐΰωΩηΗϵϱ",
    "saeiiyooiyiyooiier"
)
pre_processor_mapping = {
    "th": "ϑ",
    "Th": "θ",
    "ch": "χ",
    "Ch": "Χ",
    "ps": "ψ",
    "Ps": "Ψ",
    "Gk": "Γκ",
    "gk": "γκ",
    "gx": "γξ",
    "nx": "γξ",
    "nch":"γξ",
    "gch":"γξ",
    "B" : "Μπ",
    "Ev": "Ευ",
    "Ef": "Ευ",
    "Ey": "Ευ",
    "ev": "ευ",
    "ef": "ευ",
    "ey": "ευ",

    "Iv": "Ηυ",
    "If": "Ηυ",
    "Iy": "Ηυ",
    "iv": "ηυ",
    "if": "ηυ",
    "iy": "ηυ",

    "b" : "μπ",
    "A": "Αυ",
    "a": "αυ",
    "E": "Ευ",
    "e": "ευ",
    "O": "Ου",
    "o": "ου",
    "Oy": "Ου",
    "oy": "ου",
    "": "Υ",
    "": "υ",
    "Yi": "Υι",
    "yi": "υι",
    "Oi": "Οι",
    "oi": "οι",
    "ng": "γγ",
}



# -*- coding: utf-8 -*-

__title__ = 'transliterate.contrib.languages.ru.translit_language_pack'
__author__ = 'Artur Barseghyan'
__copyright__ = '2013-2015 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'
__all__ = ('RussianLanguagePack',)

from transliterate.base import TranslitLanguagePack, registry
from transliterate.contrib.languages.ru import data

class RussianLanguagePack(TranslitLanguagePack):
    """
    Language pack for Russian language. See
    `http://en.wikipedia.org/wiki/Russian_alphabet` for details.
    """
    language_code = "ru"
    language_name = "Russian"
    character_ranges = ((0x0400, 0x04FF), (0x0500, 0x052F))
    mapping = data.mapping
    reversed_specific_mapping = data.reversed_specific_mapping
    pre_processor_mapping = data.pre_processor_mapping
    detectable = True


registry.register(RussianLanguagePack)



# -*- coding: utf-8 -*-

__title__ = 'transliterate.tests.test_transliterate'
__author__ = 'Artur Barseghyan'
__copyright__ = '2013-2015 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'
__all__ = ('TransliterateTest',)

import unittest

from transliterate.discover import autodiscover
from transliterate.conf import (
    set_setting, get_setting, reset_to_defaults_settings
)
from transliterate import defaults
from transliterate import (
    get_available_language_codes, translit, detect_language, slugify
)
from transliterate import get_available_language_packs
from transliterate.decorators import (
    transliterate_function, transliterate_method
)
from transliterate.base import TranslitLanguagePack, registry

from transliterate.contrib.apps.translipsum import TranslipsumGenerator

from transliterate.tests.helpers import print_info
from transliterate.tests import data

class TransliterateTest(unittest.TestCase):
    """
    Tests of ``transliterate.utils.translit``.
    """
    def setUp(self):
        self.latin_text = data.latin_text
        self.armenian_text = data.armenian_text
        self.cyrillic_text = data.cyrillic_text
        self.ukrainian_cyrillic_text = data.ukrainian_cyrillic_text
        self.bulgarian_cyrillic_text = data.bulgarian_cyrillic_text
        self.georgian_text = data.georgian_text
        self.greek_text = data.greek_text
        self.hebrew_text = data.hebrew_text
        self.mongolian_cyrillic_text = data.mongolian_cyrillic_text
        #reset_to_defaults_settings()

    @print_info
    def test_01_get_available_language_codes(self):
        """
        Test ``autodiscover`` and ``get_available_language_codes``.
        """
        res = get_available_language_codes()
        res.sort()
        c = ['el', 'hy', 'ka', 'ru', 'uk', 'bg','mk', 'mn'] #'he',
        c.sort()
        self.assertEqual(res, c)
        return res

    @print_info
    def test_02_translit_latin_to_armenian(self):
        """
        Test transliteration from Latin to Armenian.
        """
        res = translit(self.latin_text, 'hy')
        self.assertEqual(res, self.armenian_text)
        return res

    @print_info
    def test_03_translit_latin_to_georgian(self):
        """
        Test transliteration from Latin to Georgian.
        """
        res = translit(self.latin_text, 'ka')
        self.assertEqual(res, self.georgian_text)
        return res

    @print_info
    def test_04_translit_latin_to_greek(self):
        """
        Test transliteration from Latin to Greek.
        """
        res = translit(self.latin_text, 'el')
        self.assertEqual(res, self.greek_text)
        return res

    @print_info
    def __test_05_translit_latin_to_hebrew(self):
        """
        Test transliteration from Latin to Hebrew.
        """
        res = translit(self.latin_text, 'he')
        self.assertEqual(res, self.hebrew_text)
        return res

    @print_info
    def test_06_translit_latin_to_cyrillic(self):
        """
        Test transliteration from Latin to Cyrillic.
        """
        res = translit(self.latin_text, 'ru')
        self.assertEqual(res, self.cyrillic_text)
        return res

    @print_info
    def test_06_translit_latin_to_ukrainian_cyrillic(self):
        """
        Test transliteration from Latin to Ukrainian Cyrillic.
        """
        res = translit(self.latin_text, 'uk')
        self.assertEqual(res, self.ukrainian_cyrillic_text)
        return res

    @print_info
    def test_06_translit_latin_to_bulgarian_cyrillic(self):
        """
        Test transliteration from Latin to Bulgarian Cyrillic.
        """
        res = translit(self.latin_text, 'bg')
        self.assertEqual(res, self.bulgarian_cyrillic_text)
        return res

    @print_info
    def test_06_translit_latin_to_mongolian_cyrillic(self):
        """
        Test transliteration from Latin to Mongolian Cyrillic.
        """
        res = translit(self.latin_text, 'mn')
        self.assertEqual(res, self.mongolian_cyrillic_text)
        return res

    @print_info
    def test_07_translit_armenian_to_latin(self):
        """
        Test transliteration from Armenian to Latin.
        """
        res = translit(self.armenian_text, 'hy', reversed=True)
        self.assertEqual(res, self.latin_text)
        return res

    @print_info
    def test_08_translit_georgian_to_latin(self):
        """
        Test transliteration from Georgian to Latin.
        """
        res = translit(self.georgian_text, 'ka', reversed=True)
        self.assertEqual(res, self.latin_text)
        return res

    @print_info
    def test_09_translit_greek_to_latin(self):
        """
        Test transliteration from Greek to Latin.
        """
        res = translit(self.greek_text, 'el', reversed=True)
        self.assertEqual(res, self.latin_text)
        return res

    @print_info
    def __test_10_translit_hebrew_to_latin(self):
        """
        Test transliteration from Hebrew to Latin.
        """
        res = translit(self.hebrew_text, 'he', reversed=True)
        self.assertEqual(res, self.latin_text)
        return res

    @print_info
    def test_11_translit_cyrillic_to_latin(self):
        """
        Test transliteration from Cyrillic to Latin.
        """
        res = translit(self.cyrillic_text, 'ru', reversed=True)
        self.assertEqual(res, self.latin_text)
        return res

    @print_info
    def test_11_translit_ukrainian_cyrillic_to_latin(self):
        """
        Test transliteration from Ukrainian Cyrillic to Latin.
        """
        res = translit(self.ukrainian_cyrillic_text, 'uk', reversed=True)
        self.assertEqual(res, self.latin_text)
        return res

    @print_info
    def test_11_translit_bulgarian_cyrillic_to_latin(self):
        """
        Test transliteration from Bulgarian Cyrillic to Latin.
        """
        res = translit(self.bulgarian_cyrillic_text, 'bg', reversed=True)
        self.assertEqual(res, self.latin_text)
        return res

    @print_info
    def test_11_translit_mongolian_cyrillic_to_latin(self):
        """
        Test transliteration from Mongolian Cyrillic to Latin.
        """
        res = translit(self.mongolian_cyrillic_text, 'mn', reversed=True)
        self.assertEqual(res, self.latin_text)
        return res

    @print_info
    def test_12_function_decorator(self):
        """
        Testing the function decorator from Latin to Armenian.
        """
        @transliterate_function(language_code='hy')
        def decorator_test_armenian(text):
            return text

        res = decorator_test_armenian(self.latin_text)
        self.assertEqual(res, self.armenian_text)

    @print_info
    def test_13_method_decorator(self):
        """
        Testing the method decorator from Latin to Cyrillic.
        """
        class DecoratorTest(object):
            @transliterate_method(language_code='ru')
            def decorator_test_russian(self, text):
                return text

        res = DecoratorTest().decorator_test_russian(self.latin_text)
        self.assertEqual(res, self.cyrillic_text)
        return res

    @print_info
    def test_14_function_decorator(self):
        """
        Testing the function decorator (reversed) from Armenian to Latin.
        """
        @transliterate_function(language_code='hy', reversed=True)
        def decorator_test_armenian_reversed(text):
            return text

        res = decorator_test_armenian_reversed(self.armenian_text)
        self.assertEqual(res, self.latin_text)
        return res

    @print_info
    def test_15_register_custom_language_pack(self):
        """
        Testing registering of a custom language pack.
        """
        class ExampleLanguagePack(TranslitLanguagePack):
            """
            Example language pack.
            """
            language_code = "example"
            language_name = "Example"
            mapping = data.test_15_register_custom_language_pack_mapping

        registry.register(ExampleLanguagePack)

        assert 'example' in get_available_language_codes()
        res = translit(self.latin_text, 'example')
        self.assertEqual(res, 'Lor5m 9psum 4olor s9t 1m5t')
        return res

    @print_info
    def test_16_translipsum_generator_armenian(self):
        """
        Testing the translipsum generator. Generating lorem ipsum paragraphs
        in Armenian.
        """
        g_am = TranslipsumGenerator(language_code='hy')
        res = g_am.generate_paragraph()
        assert res
        return res

    @print_info
    def test_17_translipsum_generator_georgian(self):
        """
        Testing the translipsum generator. Generating lorem ipsum sentence
        in Georgian.
        """
        g_ge = TranslipsumGenerator(language_code='ka')
        res = g_ge.generate_sentence()
        assert res
        return res

    @print_info
    def test_18_translipsum_generator_greek(self):
        """
        Testing the translipsum generator. Generating lorem ipsum sentence
        in Greek.
        """
        g_el = TranslipsumGenerator(language_code='el')
        res = g_el.generate_sentence()
        assert res
        return res

    @print_info
    def __test_19_translipsum_generator_hebrew(self):
        """
        Testing the translipsum generator. Generating lorem ipsum sentence
        in Hebrew.
        """
        g_he = TranslipsumGenerator(language_code='he')
        res = g_he.generate_sentence()
        assert res
        return res

    @print_info
    def test_20_translipsum_generator_cyrillic(self):
        """
        Testing the translipsum generator. Generating lorem ipsum sentence
        in Cyrillic.
        """
        g_ru = TranslipsumGenerator(language_code='ru')
        res = g_ru.generate_sentence()
        assert res
        return res

    @print_info
    def test_20_translipsum_generator_ukrainian_cyrillic(self):
        """
        Testing the translipsum generator. Generating lorem ipsum sentence
        in Ukrainian Cyrillic.
        """
        g_uk = TranslipsumGenerator(language_code='uk')
        res = g_uk.generate_sentence()
        assert res
        return res

    @print_info
    def test_20_translipsum_generator_bulgarian_cyrillic(self):
        """
        Testing the translipsum generator. Generating lorem ipsum sentence
        in Bulgarian Cyrillic.
        """
        g_bg = TranslipsumGenerator(language_code='bg')
        res = g_bg.generate_sentence()
        assert res
        return res

    @print_info
    def test_20_translipsum_generator_mongolian_cyrillic(self):
        """
        Testing the translipsum generator. Generating lorem ipsum sentence
        in Mongolian Cyrillic.
        """
        g_bg = TranslipsumGenerator(language_code='mn')
        res = g_bg.generate_sentence()
        assert res
        return res

    @print_info
    def test_21_language_detection_armenian(self):
        """
        Testing language detection. Detecting Amenian.
        """
        res = detect_language(self.armenian_text)
        self.assertEqual(res, 'hy')
        return res

    @print_info
    def test_22_language_detection_georgian(self):
        """
        Testing language detection. Detecting Georgian.
        """
        res = detect_language(self.georgian_text)
        self.assertEqual(res, 'ka')
        return res

    @print_info
    def test_23_language_detection_greek(self):
        """
        Testing language detection. Detecting Greek.
        """
        #set_setting('DEBUG', True)
        res = detect_language(self.greek_text)
        #reset_to_defaults_settings()
        self.assertEqual(res, 'el')
        return res

    @print_info
    def __test_24_language_detection_hebrew(self):
        """
        Testing language detection. Detecting Hebrew.
        """
        res = detect_language(self.hebrew_text)
        self.assertEqual(res, 'he')
        return res

    @print_info
    def test_25_language_detection_cyrillic(self):
        """
        Testing language detection. Detecting Russian (Cyrillic).
        """
        res = detect_language(self.cyrillic_text)
        self.assertEqual(res, 'ru')
        return res

    @print_info
    def test_25_false_language_detection_cyrillic(self):
        """
        Testing language detection. Detecting is not Russian (Cyrillic).
        """
        res = detect_language(self.latin_text)
        self.assertNotEqual(res, 'ru')
        return res

    @print_info
    def __test_25_language_detection_ukrainian_cyrillic(self):
        """
        Testing language detection. Detecting Ukrainian (Cyrillic).
        """
        res = detect_language(self.ukrainian_cyrillic_text)
        self.assertEqual(res, 'uk')
        return res

    @print_info
    def __test_25_language_detection_bulgarian_cyrillic(self):
        """
        Testing language detection. Detecting Bulgarian (Cyrillic).
        """
        res = detect_language(self.bulgarian_cyrillic_text)
        self.assertEqual(res, 'bg')
        return res

    @print_info
    def __test_25_language_detection_mongolian_cyrillic(self):
        """
        Testing language detection. Detecting Mongolian (Cyrillic).
        """
        res = detect_language(self.mongolian_cyrillic_text)
        self.assertEqual(res, 'mn')
        return res

    @print_info
    def test_26_slugify_armenian(self):
        """
        Testing slugify from Armenian.
        """
        res = slugify(self.armenian_text)
        self.assertEqual(res, 'lorem-ipsum-dolor-sit-amet')
        return res

    @print_info
    def test_27_slugify_georgian(self):
        """
        Testing slugify from Georgian.
        """
        res = slugify(self.georgian_text)
        self.assertEqual(res, 'lorem-ipsum-dolor-sit-amet')
        return res

    @print_info
    def test_28_slugify_greek(self):
        """
        Testing slugify from Greek.
        """
        res = slugify(self.greek_text)
        self.assertEqual(res, 'lorem-ipsum-dolor-sit-amet')
        return res

    @print_info
    def __test_29_slugify_hebrew(self):
        """
        Testing slugify from Hebrew.
        """
        res = slugify(self.hebrew_text)
        self.assertEqual(res, 'lorem-ipsum-dolor-sit-amet')
        return res

    @print_info
    def test_30_slugify_cyrillic(self):
        """
        Testing slugify from Cyrillic.
        """
        res = slugify(self.cyrillic_text)
        self.assertEqual(res, 'lorem-ipsum-dolor-sit-amet')
        return res

    @print_info
    def test_30_slugify_ukrainian_cyrillic(self):
        """
        Testing slugify from Ukrainian Cyrillic.
        """
        res = slugify(self.ukrainian_cyrillic_text, language_code='uk')
        self.assertEqual(res, 'lorem-ipsum-dolor-sit-amet')
        return res

    @print_info
    def test_30_slugify_bulgarian_cyrillic(self):
        """
        Testing slugify from Bulgarian Cyrillic.
        """
        res = slugify(self.bulgarian_cyrillic_text, language_code='bg')
        self.assertEqual(res, 'lorem-ipsum-dolor-sit-amet')
        return res

    @print_info
    def test_30_slugify_mongolian_cyrillic(self):
        """
        Testing slugify from Mongolian Cyrillic.
        """
        res = slugify(self.mongolian_cyrillic_text, language_code='mn')
        self.assertEqual(res, 'lorem-ipsum-dolor-sit-amet')
        return res

    @print_info
    def test_31_override_settings(self):
        """
        Testing settings override.
        """
        def override_settings():
            return get_setting('LANGUAGE_DETECTION_MAX_NUM_KEYWORDS')

        self.assertEqual(defaults.LANGUAGE_DETECTION_MAX_NUM_KEYWORDS,
                         override_settings())

        set_setting('LANGUAGE_DETECTION_MAX_NUM_KEYWORDS', 10)

        self.assertEqual(10, override_settings())

        return override_settings()

    @print_info
    def test_32_auto_translit_reversed(self):
        """
        Test automatic reversed translit (from target script to source script)
        for Armenian, Georgian, Greek and Russian (Cyrillic).
        """
        res = []
        texts = [
            self.armenian_text,
            self.georgian_text,
            self.greek_text,
            #self.hebrew_text,
            self.cyrillic_text
        ]

        for text in texts:
            r = translit(text, reversed=True)
            self.assertEqual(r, self.latin_text)
            res.append(r)

        return res

    @print_info
    def test_33_register_unregister(self):
        """
        Testing register/unregister.
        """
        from transliterate.contrib.languages.hy.translit_language_pack import (
            ArmenianLanguagePack
        )

        class A(TranslitLanguagePack):
            language_code = "ru"
            language_name = "Example"
            mapping = data.test_33_register_unregister_mapping
        # Since key `ru` already exists in the registry it can't be replaced
        # (without force-register).
        res = registry.register(A)
        self.assertTrue(not res)

        # Now with force-register it can.
        res = registry.register(A, force=True)
        self.assertTrue(res)

        # Once we have it there and it's forced, we can't register another.
        res = registry.register(A, force=True)
        self.assertTrue(not res)

        # Unregister non-forced language pack.
        res = registry.unregister(ArmenianLanguagePack)
        self.assertTrue(
            res and not ArmenianLanguagePack.language_code in
                        get_available_language_codes()
        )

        res = registry.unregister(A)
        self.assertTrue(
            not res and A.language_code in get_available_language_codes()
        )

    @print_info
    def __test_34_latin_to_latin(self):

        class LatinToLatinLanguagePack(TranslitLanguagePack):
            """
            Custom language pack which gets rid of accented characters in Greek
            but leaves other characters intact.
            """
            language_code = "l2l"
            language_name = "Latin to Latin"
            mapping = data.test_34_latin_to_latin_mapping
            characters = data.test_34_latin_to_latin_characters
            reversed_characters = \
                data.test_34_latin_to_latin_reversed_characters

        res = registry.register(LatinToLatinLanguagePack)
        self.assertTrue(res)

        text = data.test_34_latin_to_latin_text
        pack = LatinToLatinLanguagePack()
        res = pack.translit(text, strict=True, fail_silently=False)

    @print_info
    def __test_29_mappings(self):
        """
        Testing mappings.
        """
        for language_pack in get_available_language_packs():
            print_('Testing language '
                   'pack {0} {1}'.format(language_pack.language_code,
                                         language_pack.language_name))
            print_('Reversed test:')
            for letter in language_pack.mapping[1]:
                print_(letter, ' --> ', translit(letter,
                                                 language_pack.language_code,
                                                 reversed=True))

            print_('Normal test:')
            for letter in language_pack.mapping[0]:
                print_(letter, ' --> ', translit(letter,
                                                 language_pack.language_code))


if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-

__title__ = 'transliterate.contrib.languages.el.translit_language_pack'
__author__ = 'Artur Barseghyan'
__copyright__ = '2013-2015 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'
__all__ = ('GreekLanguagePack',)

from transliterate.base import TranslitLanguagePack, registry
from transliterate.contrib.languages.el import data

class GreekLanguagePack(TranslitLanguagePack):
    """
    Language pack for Greek language. See
    `http://en.wikipedia.org/wiki/Greek_alphabet` for details.
    """
    language_code = "el"
    language_name = "Greek"
    character_ranges = ((0x0370, 0x03FF), (0x1F00, 0x1FFF))
    mapping = data.mapping
    reversed_specific_mapping = data.reversed_specific_mapping
    pre_processor_mapping = data.pre_processor_mapping
    detectable = True


registry.register(GreekLanguagePack)



__title__ = 'transliterate.contrib.apps.translipsum.__init__'
__author__ = 'Artur Barseghyan'
__copyright__ = '2013-2015 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'
__all__ = ('TranslipsumGenerator',)

import logging
logger = logging.getLogger(__name__)

from six import PY3

if PY3:
    from transliterate.contrib.apps.translipsum.utils import Generator
else:
    from lipsum import Generator

from transliterate.utils import translit

class TranslipsumGenerator(Generator):
    """
    Lorem ipsum generator.
    """
    def __init__(self, language_code, reversed=False, *args, **kwargs):
        self._language_code = language_code
        self._reversed = reversed
        super(TranslipsumGenerator, self).__init__(*args, **kwargs)

    def generate_sentence(self, *args, **kwargs):
        value = super(TranslipsumGenerator, self).generate_sentence(
            *args, **kwargs
        )
        return translit(value,
                        language_code=self._language_code,
                        reversed=self._reversed)

    def generate_paragraph(self, *args, **kwargs):
        value = super(TranslipsumGenerator, self).generate_paragraph(
            *args, **kwargs
        )
        return translit(value,
                        language_code=self._language_code,
                        reversed=self._reversed)



# -*- coding: utf-8 -*-

mapping = (
    u"avgdeziklmnxoprstyfAVGDEZIKLMNXOPRSTYF",
    u"αβγδεζικλμνξοπρστυφΑΒΓΔΕΖΙΚΛΜΝΞΟΠΡΣΤΥΦ",
)
reversed_specific_mapping = (
    u"ςάέήίύόώϊϋΐΰωΩηΗϵϱ",
    u"saeiiyooiyiyooiier"
)
pre_processor_mapping = {
    u"th": u"ϑ",
    u"Th": u"θ",
    u"ch": u"χ",
    u"Ch": u"Χ",
    u"ps": u"ψ",
    u"Ps": u"Ψ",
    u"Gk": u"Γκ",
    u"gk": u"γκ",
    u"gx": u"γξ",
    u"nx": u"γξ",
    u"nch":u"γξ",
    u"gch":u"γξ",
    u"B" : u"Μπ",
    u"Ev": u"Ευ",
    u"Ef": u"Ευ",
    u"Ey": u"Ευ",
    u"ev": u"ευ",
    u"ef": u"ευ",
    u"ey": u"ευ",

    u"Iv": u"Ηυ",
    u"If": u"Ηυ",
    u"Iy": u"Ηυ",
    u"iv": u"ηυ",
    u"if": u"ηυ",
    u"iy": u"ηυ",

    u"b" : u"μπ",
    u"Au": u"Αυ",
    u"au": u"αυ",
    u"Eu": u"Ευ",
    u"eu": u"ευ",
    u"Ou": u"Ου",
    u"ou": u"ου",
    u"Oy": u"Ου",
    u"oy": u"ου",
    u"U": u"Υ",
    u"u": u"υ",
    u"Yi": u"Υι",
    u"yi": u"υι",
    u"Oi": u"Οι",
    u"oi": u"οι",
    u"ng": u"γγ",
}



__title__ = 'transliterate.contrib.apps.translipsum.utils'
__author__ = 'Artur Barseghyan'
__copyright__ = '2013-2015 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'
__all__ = ('Generator',)

import re
import random

from six import PY3
from six import text_type

if not PY3:
    from string import translate, maketrans, punctuation
else:
    from string import punctuation

FACTORY = """
    Sed dictum in tellus non iaculis. Aenean ac interdum ipsum. Etiam tempor
    quis ante vel rhoncus. Nulla facilisi. Curabitur iaculis consequat odio ut
    imperdiet? Integer accumsan; nisl vitae fermentum malesuada, sapien nulla
    sodales orci, et elementum lacus purus vel purus! Nullam orci neque,
    tristique in porta id, pretium ac sem. Fusce non est risus. Fusce convallis
    tellus augue, quis volutpat tellus dapibus sagittis. Integer lacinia
    commodo risus vel cursus. Etiam vitae dui in dolor porta luctus sed id
    elit. Nulla et est nec magna facilisis sagittis. Praesent tincidunt dictum
    lectus, sed aliquam eros. Donec placerat tortor ut lorem facilisis congue.
    Quisque ac risus nibh. Etiam ultrices nibh justo; sed mollis ipsum dapibus
    vitae. Ut vitae molestie erat. Mauris ac justo quis ante posuere vehicula.
    Vivamus accumsan mi volutpat diam lacinia, vitae semper lectus pharetra.
    Cras ultrices arcu nec viverra consectetur. Cras placerat ante quis dui
    consequat cursus. Nulla at enim dictum, consectetur ligula eget, vehicula
    nisi. Suspendisse eu ligula vitae est tristique accumsan nec adipiscing
    risus. Donec tempus dui eget mollis fringilla. Fusce eleifend lacus lectus,
    vel ornare felis lacinia ut. Morbi vel adipiscing augue. Vestibulum ante
    ipsum primis in faucibus orci luctus et ultrices posuere cubilia Curae;
    Cras mattis pulvinar lacus, vitae pulvinar magna egestas non. Aliquam in
    urna quis leo feugiat faucibus. Aliquam erat volutpat. Maecenas non mauris
    libero. Suspendisse nisi lorem, cursus a tristique a, porttitor in nisl.
    Mauris pellentesque gravida mi non mattis. Cras mauris ligula, interdum
    semper tincidunt sed, ornare a ipsum. Nulla ultrices tempus tortor vitae
    vehicula. Etiam at augue suscipit, vehicula sapien sit amet; eleifend orci.
    Etiam venenatis leo nec cursus mattis. Nulla suscipit nec lorem et
    lobortis. Donec interdum vehicula massa sed aliquam. Praesent eleifend mi
    sed mi pretium pellentesque. In in nisi tincidunt, commodo lorem quis;
    tincidunt nisl. In suscipit quam a vehicula tincidunt! Fusce vitae varius
    nunc. Proin at ipsum ac tellus hendrerit ultricies. Phasellus auctor
    hendrerit sapien viverra facilisis. Suspendisse lacus erat, cursus at dolor
    in, vulputate convallis sapien. Etiam augue nunc, lobortis vel viverra sit
    amet, pretium et lacus. Pellentesque elementum lectus eget massa tempus
    elementum? Nulla nec auctor dolor. Aliquam congue purus quis libero
    fermentum cursus. Etiam quis massa ac nisl accumsan convallis vitae ac
    augue. Mauris neque est, posuere quis dolor non, volutpat gravida tortor.
    Cum sociis natoque penatibus et magnis dis parturient montes, nascetur
    ridiculus mus. Vivamus ullamcorper, urna at ultrices aliquam, orci libero
    gravida ligula, non pulvinar sem magna sed tortor. Sed elementum leo
    viverra ipsum aliquet convallis. Suspendisse scelerisque auctor sapien.
    Mauris enim nisl, sollicitudin at rhoncus vitae, convallis nec mauris.
    Phasellus sollicitudin dui ut luctus consectetur. Vivamus placerat, neque
    id sagittis porttitor, nunc quam varius dolor, sit amet egestas nulla
    risus eu odio. Mauris gravida eleifend laoreet. Aenean a nulla nisl.
    Integer pharetra magna adipiscing, imperdiet augue ac, blandit felis. Cras
    id aliquam neque, vel consequat sapien.Duis eget vulputate ligula. Aliquam
    ornare dui non nunc laoreet, non viverra dolor semper. Aenean ullamcorper
    velit sit amet dignissim fermentum! Aenean urna leo, rutrum volutpat mauris
    nec, facilisis molestie tortor. In convallis pellentesque lorem, a lobortis
    erat molestie et! Ut sed sem a odio aliquam elementum. Morbi pretium velit
    libero, adipiscing consequat leo dignissim eu. Mauris vestibulum feugiat
    risus; quis pharetra purus tincidunt quis. Morbi semper tincidunt lorem id
    iaculis. Quisque non pulvinar magna. Morbi consequat eleifend neque et
    iaculis. Fusce non laoreet urna. Donec ut nunc ultrices, fringilla nunc ut,
    tempor elit. Phasellus semper sapien augue, in gravida neque egestas at.
    Integer dapibus lacus vitae luctus sagittis! Suspendisse imperdiet tortor
    eget mattis consectetur. Aliquam viverra purus a quam lacinia euismod.
    Nunc non consequat mi; ac vehicula lacus. Pellentesque accumsan ac diam in
    fermentum! Maecenas quis nibh sed dolor adipiscing facilisis. Aenean vel
    arcu eu est fermentum egestas vulputate eget purus. Sed fermentum rhoncus
    dapibus. Quisque molestie magna eu accumsan lobortis. Vestibulum cursus
    euismod posuere. Aliquam eu dapibus urna. Nulla id accumsan justo. Vivamus
    vitae ullamcorper tellus. Class aptent taciti sociosqu ad litora torquent
    per conubia nostra, per inceptos himenaeos.Donec pulvinar tempus lectus
    vitae ultricies. Vestibulum sagittis orci quis risus ultricies feugiat.
    Nunc feugiat velit est, at aliquam massa tristique eu. Aenean quis enim
    vel leo vestibulum volutpat in non elit. Quisque molestie tincidunt purus;
    ac lacinia mauris rhoncus in. Nullam id arcu at mauris varius viverra ut
    vitae massa. In ac nunc ipsum. Proin consectetur urna sit amet mattis
    vulputate. Nullam lacinia pretium tempus. Aenean quis ornare metus, tempus
    volutpat neque. Mauris volutpat scelerisque augue; at lobortis nulla
    rhoncus vitae. Mauris at lobortis turpis. Vivamus et ultrices lacus.
    Donec fermentum neque in eros cursus, ac tincidunt sapien consequat.
    Curabitur varius commodo rutrum. Nulla facilisi. Ut feugiat dui nec turpis
    sodales aliquam. Quisque auctor vestibulum condimentum. Quisque nec eros
    lorem. Curabitur et felis nec diam dictum ultrices vestibulum ac eros!
    Quisque eu pretium lacus. Morbi bibendum sagittis rutrum. Nam eget tellus
    quam. Nullam pharetra vestibulum justo. Donec molestie urna et scelerisque
    laoreet? Sed consectetur pretium hendrerit. Quisque erat nulla, elementum
    sit amet nibh vel, posuere pulvinar nulla. Donec elementum adipiscing
    dictum! Nam euismod semper nisi, eu lacinia felis placerat vel! Praesent
    eget dapibus turpis, et fringilla elit. Maecenas quis nunc cursus felis
    fringilla consequat! Cum sociis natoque penatibus et magnis dis parturient
    montes, nascetur ridiculus mus. Sed ullamcorper libero quis nisl
    sollicitudin, ut pulvinar arcu consectetur. Donec nisi nibh, condimentum
    et lectus non, accumsan imperdiet ipsum. Maecenas vitae massa eget lorem
    ornare dignissim. Nullam condimentum mauris id quam tincidunt venenatis.
    Aenean mattis viverra sem, vitae luctus velit rhoncus non. Vestibulum leo
    justo, rhoncus at aliquam et, iaculis sed dolor. Integer bibendum vitae
    urna in ornare! Cras accumsan nulla eu libero tempus, in dignissim augue
    imperdiet. Vivamus a lacinia odio. Curabitur id egestas eros. Integer non
    rutrum est. In nibh sem, tempus ac dignissim vel, ornare ac mi. Nulla
    congue scelerisque est nec commodo. Phasellus turpis lorem, sodales quis
    sem id, facilisis commodo massa. Vestibulum ultrices dolor eget purus
    semper euismod? Fusce id congue leo. Quisque dui magna, ullamcorper et leo
    eget, commodo facilisis ipsum. Curabitur congue vitae risus nec posuere.
    Phasellus tempor ligula in nisl pellentesque mattis. Sed nunc turpis,
    pharetra vel leo ac, lacinia cursus risus. Quisque congue aliquet volutpat.
    Integer dictum est quis semper tristique. Donec feugiat vestibulum tortor,
    id fringilla nisi lobortis eu. Nam hendrerit egestas sem, non mollis tortor
    iaculis quis. Phasellus id aliquet erat. Nunc facilisis nisi dolor, quis
    semper dui euismod vel. Cras convallis bibendum tortor malesuada tincidunt.
    Sed urna quam, pellentesque eget eleifend ac, consequat bibendum urna. Sed
    fringilla elit hendrerit leo blandit laoreet eget quis quam! Morbi eu leo a
    dolor aliquet dictum. Suspendisse condimentum mauris non ipsum rhoncus, sit
    amet hendrerit augue gravida. Quisque facilisis pharetra felis faucibus
    gravida. In arcu neque, gravida ut fermentum ut, placerat eu quam. Nullam
    aliquet lectus mauris, quis dignissim est mollis sed. Ut vestibulum laoreet
    eros quis cursus. Proin commodo eros in mollis mollis. Mauris bibendum
    cursus nibh, sit amet eleifend mauris luctus vitae. Sed aliquet pretium
    tristique. Morbi ultricies augue a lacinia porta. Nullam mollis erat non
    imperdiet imperdiet. Etiam tincidunt fringilla ligula, in adipiscing libero
    viverra eu. Nunc gravida hendrerit massa, in pellentesque nunc dictum id.
    """

# Splits words
if not PY3:
    split_words = lambda f: list(
        set(translate(f.lower(),
                      maketrans(punctuation, ' ' * len(punctuation))).split())
    )
else:
    split_words = lambda f: list(
        set(f.lower().translate(str.maketrans("", "", punctuation)).split())
    )

def split(delimiters, value, max_split=0):
    """
    Splits the value given by delimiters provided.

    :param str delimiters:
    :param str value:
    :param int max_split:
    :return list:
    """
    value = re.sub('\s+', ' ', value)
    pattern = '|'.join(map(re.escape, delimiters))
    return re.split(pattern, value, max_split)

# Split sentences
split_sentences = lambda f: split('!;?.', f)

WORDS = split_words(FACTORY)
SENTENCES = split_sentences(FACTORY)

class Generator(object):
    """
    Fallback lorem ipsum generator for Python 3.
    """
    def __init__(self, *args, **kwargs):
        pass

    def generate_word(self):
        return WORDS[random.randint(0, len(WORDS) - 1)]

    def generate_sentence(self):
        return text_type(SENTENCES[random.randint(0, len(SENTENCES) - 1)] + '.')

    def generate_paragraph(self, num_sentences=4):
        buffer = []
        for i in range(0, 4):
            buffer.append(self.generate_sentence())
        return ''.join(buffer)



# -*- coding: utf-8 -*-

mapping = (
    "abvgdeziyklmnoprstufhABVGDEZIYKLMNOPRSTUFH",
    "абвгдезийклмнопрстуфхАБВГДЕЗИЙКЛМНОПРСТУФХ",
)

reversed_specific_mapping = (
    "ьъЪ",
    "yaA"
)

pre_processor_mapping = {
    "zh": "ж",
    "ts": "ц",
    "ch": "ч",
    "sh": "ш",
    "sht": "щ",
    "yu": "ю",
    "ya": "я",
    "Zh": "Ж",
    "Ts": "Ц",
    "Ch": "Ч",
    "Sh": "Ш",
    "Sht": "Щ",
    "Yu": "Ю",
    "Ya": "Я"
}



# -*- coding: utf-8 -*-

mapping = (
    u"abvgdeziyklmnoprstufhABVGDEZIYKLMNOPRSTUFH",
    u"абвгдезийклмнопрстуфхАБВГДЕЗИЙКЛМНОПРСТУФХ",
)

reversed_specific_mapping = (
    u"ьъЪ",
    u"yaA"
)

pre_processor_mapping = {
    u"zh": u"ж",
    u"ts": u"ц",
    u"ch": u"ч",
    u"sh": u"ш",
    u"sht": u"щ",
    u"yu": u"ю",
    u"ya": u"я",
    u"Zh": u"Ж",
    u"Ts": u"Ц",
    u"Ch": u"Ч",
    u"Sh": u"Ш",
    u"Sht": u"Щ",
    u"Yu": u"Ю",
    u"Ya": u"Я"
}



__title__ = 'transliterate.exceptions'
__author__ = 'Artur Barseghyan'
__copyright__ = '2013-2015 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'
__all__ = (
    'LanguageCodeError', 'ImproperlyConfigured', 'LanguagePackNotFound',
    'LanguageDetectionError', 'InvalidRegistryItemType'
)

class LanguageCodeError(Exception):
    """
    Exception raised when language code is left empty or has incorrect value.
    """


class ImproperlyConfigured(Exception):
    """
    Exception raised when developer didn't configure the code properly.
    """


class LanguagePackNotFound(Exception):
    """
    Exception raised when language pack is not found for the language code
    given.
    """


class LanguageDetectionError(Exception):
    """
    Exception raised when language can't be detected for the text given.
    """


class InvalidRegistryItemType(ValueError):
    """
    Raised when an attempt is made to register an item in the registry which
    does not have a proper type.
    """



# -*- coding: utf-8 -*-

mapping = (
        u"abgdezilxkhmjnpsvtrcq&ofABGDEZILXKHMJNPSVTRCQOF",
        u"աբգդեզիլխկհմյնպսվտրցքևօֆԱԲԳԴԵԶԻԼԽԿՀՄՅՆՊՍՎՏՐՑՔՕՖ",
    )
reversed_specific_mapping = (
    u"ռՌ",
    u"rR"
)
reversed_specific_pre_processor_mapping = {
    u"ու": u"u",
    u"Ու": u"U"
}
pre_processor_mapping = {
    # lowercase
    u"e'": u"է",
    u"y": u"ը",
    u"th": u"թ",
    u"jh": u"ժ",
    u"ts": u"ծ",
    u"dz": u"ձ",
    u"gh": u"ղ",
    u"tch": u"ճ",
    u"sh": u"շ",
    u"vo": u"ո",
    u"ch": u"չ",
    u"dj": u"ջ",
    u"ph": u"փ",
    u"u": u"ու",

    # uppercase
    u"E'": u"Է",
    u"Y": u"Ը",
    u"Th": u"Թ",
    u"Jh": u"Ժ",
    u"Ts": u"Ծ",
    u"Dz": u"Ձ",
    u"Gh": u"Ղ",
    u"Tch": u"Ճ",
    u"Sh": u"Շ",
    u"Vo": u"Ո",
    u"Ch": u"Չ",
    u"Dj": u"Ջ",
    u"Ph": u"Փ",
    u"U": u"Ու"
}


# -*- coding: utf-8 -*-

mapping = (
    "abvgdezijklmnoprstufhcABVGDEZIJKLMNOPRSTUFHC",
    "абвгдезијклмнопрстуфхцАБВГДЕЗИЈКЛМНОПРСТУФХЦ",
)

reversed_specific_mapping = (
    "",
    ""
)

pre_processor_mapping = {
    "gj": "ѓ",
    "zh": "ж",
    "dzh": "ѕ",
    "lj": "љ",
    "nj": "њ",
    "кј": "ќ",
    "ch": "ч",
    "dj": "џ",
    "sh": "ш",
    "Gj": "Ѓ",
    "Zh": "Ж",
    "Dz": "Ѕ",
    "Lj": "Љ",
    "Nj": "Њ",
    "Kj": "Ќ",
    "Ch": "Ч",
    "Dzh": "Џ",
    "Sh": "Ш",
}



# -*- coding: utf-8 -*-

mapping = (
    u"abgdezilkhmjnpsvturqoABGDEZILKHMJNPSVTURQO",
    u"აბგდეზილქჰმჯნფსვთურყოႠႡႢႣႤႦႨႪႵჀႫႿႼႴႱႥႧႳႰႷႭ",
)
pre_processor_mapping = {
    u"k'": u"კ",
    u"p'": u"პ",
    u"zh'": u"ჟ",
    u"t'": u"ტ",
    u"gh": u"ღ",
    u"sh": u"შ",
    u"ch": u"ჩ",
    u"ts": u"ც",
    u"dz": u"ძ",
    u"ts'": u"წ",
    u"ch'": u"ჭ",
    u"kh": u"ხ",
    u"K'": u"Ⴉ",
    u"P'": u"Ⴎ",
    u"Zh'": u"Ⴏ",
    u"T'": u"Ⴒ",
    u"Gh": u"Ⴖ",
    u"Sh": u"Ⴘ",
    u"Ch": u"Ⴙ",
    u"Ts": u"Ⴚ",
    u"Dz": u"Ⴛ",
    u"Ts'": u"Ⴜ",
    u"Ch'": u"Ⴝ",
    u"Kh": u"Ⴞ",
}



# -*- coding: utf-8 -*-

__title__ = 'transliterate.tests.data.python32'
__author__ = 'Artur Barseghyan'
__copyright__ = '2013-2015 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'

latin_text = "Lorem ipsum dolor sit amet"
armenian_text = 'Լօրեմ իպսում դօլօր սիտ ամետ'
cyrillic_text = 'Лорем ипсум долор сит амет'
ukrainian_cyrillic_text = 'Лорем іпсум долор сіт амет'
bulgarian_cyrillic_text = 'Лорем ипсум долор сит амет'
georgian_text = 'Ⴊორემ იფსუმ დოლორ სით ამეთ'
greek_text = 'Λορεμ ιψυμ δολορ σιτ αμετ'
hebrew_text = 'Lורeמ יpסuמ דולור סית אמeת'
mongolian_text = u'Лорэм ипсүм долор сит амет'

test_15_register_custom_language_pack_mapping = (
    "abcdefghij",
    "1234567890",
)

test_33_register_unregister_mapping = (
    "abcdefghij",
    "1234567890",
)

test_34_latin_to_latin_mapping = (
    "abgdezilxkhmjnpsvtrcqw&ofABGDEZILXKHMJNPSVTRCQOFW",
    "zbgdeailxkhnjmpswtrcqv&ofZBGDEAILXKHNJMPSWTRCQOFV",
)

test_34_latin_to_latin_characters = "abgdezilxkhmjnpsvtrcqw&ofABGDEZILXKHMJNPSVTRCQOFW"

test_34_latin_to_latin_reversed_characters = "abgdezilxkhmjnpsvtrcqw&ofABGDEZILXKHMJNPSVTRCQOFW"

test_34_latin_to_latin_text = "Lorem ipsum dolor sit amet 123453254593485938"
